#region Copyright (C) 2003-2015 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports       										}
{																	}
{	Copyright (C) 2003-2015 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2015 Stimulsoft

using System;
using System.Windows.Forms;
using System.ComponentModel;
using Stimulsoft.Base.Localization;
using Stimulsoft.Base.Serializing;
using Stimulsoft.Report.Dictionary.Design;

namespace Stimulsoft.Report.Dictionary
{

	[TypeConverter(typeof(StiSqlDatabaseConverter))]
	public class StiOracleDatabase : StiSqlDatabase
	{
		#region StiService override
		public override string ServiceName
		{
			get
			{
				return StiLocalization.Get("Database", "DatabaseOracle");
			}
		}
		#endregion

		#region DataAdapter override
		protected override string DataAdapterType
		{
			get
			{
				return "Stimulsoft.Report.Dictionary.StiOracleAdapterService";
			}
		}
		#endregion

        public override bool CanEditConnectionString
        {
            get
            {
                return true;
            }
        }

        public override string EditConnectionString(string connectionString)
        {
            OracleConnectionEditor frm = new OracleConnectionEditor(connectionString);
            if (frm.ShowDialog() == DialogResult.OK)
            {
                return frm.OracleBuilder.ToString();
            }
            return connectionString;
        }

		public override DialogResult Edit(bool newDatabase)
		{
			using (StiSqlDatabaseEditForm form = new StiSqlDatabaseEditForm(this))
			{
				if (newDatabase)form.Text = StiLocalization.Get("FormDatabaseEdit", "OracleNew");
				else form.Text = StiLocalization.Get("FormDatabaseEdit", "OracleEdit");

                form.tbName.Text = this.Name;
                form.tbAlias.Text = this.Alias;
                form.tbConnectionString.Text = this.ConnectionString;
                if (form.ShowDialog() == DialogResult.OK)
                {
                    this.Name = form.tbName.Text;
                    this.Alias = form.tbAlias.Text;
                    this.ConnectionString = form.tbConnectionString.Text;
                }
				return form.DialogResult;
			}
		}


		/// <summary>
        /// Creates a new object of the type StiOracleDatabase.
		/// </summary>
        public StiOracleDatabase()
            : this(string.Empty, string.Empty)
		{
		}


		/// <summary>
        /// Creates a new object of the type StiOracleDatabase.
		/// </summary>
        public StiOracleDatabase(string name, string connectionString)
            : base(name, connectionString)
        {
        }
		

		/// <summary>
        /// Creates a new object of the type StiOracleDatabase.
		/// </summary>
        public StiOracleDatabase(string name, string alias, string connectionString)
            : base(name, alias, connectionString)
        {
        }


        /// <summary>
		/// Creates a new object of the type StiOracleDatabase.
		/// </summary>
		public StiOracleDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword) :
            base(name, alias, connectionString, promptUserNameAndpassword)
		{
		}

        /// <summary>
        /// Creates a new object of the type StiOracleDatabase.
        /// </summary>
        public StiOracleDatabase(string name, string alias, string connectionString, bool promptUserNameAndpassword, string key) :
            base(name, alias, connectionString, promptUserNameAndpassword, key)
        {
        }	
	}

}
